@@ -12,6 +12,7 @@
 #include <opencv2/calib3d.hpp>
 #include <opencv2/core/utils/configuration.private.hpp>
 
+
 namespace cv
 {
 namespace ovis
@@ -22,7 +23,7 @@ const char* RESOURCEGROUP_NAME = "OVIS";
 Ptr<Application> _app;
 
 static const char* RENDERSYSTEM_NAME = "OpenGL 3+ Rendering Subsystem";
-static std::vector<String> _extraResourceLocations;
+static std::set<String> _extraResourceLocations;
 
 // convert from OpenCV to Ogre coordinates:
 static Quaternion toOGRE(Degree(180), Vector3::UNIT_X);
@@ -122,11 +123,11 @@ static void _setCameraIntrinsics(Camera* cam, InputArray _K, const Size& imsize)
 
     Matx33f K = _K.getMat();
 
-    float near = cam->getNearClipDistance();
-    float top = near * K(1, 2) / K(1, 1);
-    float left = -near * K(0, 2) / K(0, 0);
-    float right = near * (imsize.width - K(0, 2)) / K(0, 0);
-    float bottom = -near * (imsize.height - K(1, 2)) / K(1, 1);
+    float zNear = cam->getNearClipDistance();
+    float top = zNear * K(1, 2) / K(1, 1);
+    float left = -zNear * K(0, 2) / K(0, 0);
+    float right = zNear * (imsize.width - K(0, 2)) / K(0, 0);
+    float bottom = -zNear * (imsize.height - K(1, 2)) / K(1, 1);
 
     // use frustum extents instead of setFrustumOffset as the latter
     // assumes centered FOV, which is not the case
@@ -170,6 +171,14 @@ static SceneNode& _getSceneNode(SceneManager* sceneMgr, const String& name)
     return *mo->getParentSceneNode();
 }
 
+static ColourValue convertColor(const Scalar& val)
+{
+    // BGR 0..255 (uchar) to RGB 0..1
+    ColourValue ret = ColourValue(val[2], val[1], val[0]) / 255;
+    ret.saturate();
+    return ret;
+}
+
 struct Application : public OgreBites::ApplicationContext, public OgreBites::InputListener
 {
     Ptr<LogManager> logMgr;
@@ -181,7 +190,7 @@ struct Application : public OgreBites::ApplicationContext, public OgreBites::Inp
     int flags;
 
     Application(const Ogre::String& _title, const Size& sz, int _flags)
-        : OgreBites::ApplicationContext("ovis", false), sceneMgr(NULL), title(_title), w(sz.width),
+        : OgreBites::ApplicationContext("ovis"), sceneMgr(NULL), title(_title), w(sz.width),
           h(sz.height), key_pressed(-1), flags(_flags)
     {
         if(utils::getConfigurationParameterBool("OPENCV_OVIS_VERBOSE_LOG", false))
@@ -227,7 +236,8 @@ struct Application : public OgreBites::ApplicationContext, public OgreBites::Inp
         if (flags & SCENE_AA)
             miscParams["FSAA"] = "4";
 
-        miscParams["vsync"] = "true";
+        miscParams["vsync"] = Ogre::StringConverter::toString(
+            !utils::getConfigurationParameterBool("OPENCV_OVIS_NOVSYNC", false));
 
         OgreBites::NativeWindowPair ret =
             OgreBites::ApplicationContext::createWindow(_name, _w, _h, miscParams);
@@ -236,6 +246,20 @@ struct Application : public OgreBites::ApplicationContext, public OgreBites::Inp
         return ret;
     }
 
+#if OGRE_VERSION < ((1 << 16) | (12 << 8) | 3)
+    void destroyWindow(const Ogre::String& name)
+    {
+        for (auto it = mWindows.begin(); it != mWindows.end(); ++it)
+        {
+            if (it->render->getName() != name)
+                continue;
+            mRoot->destroyRenderTarget(it->render);
+            mWindows.erase(it);
+            return;
+        }
+    }
+#endif
+
     size_t numWindows() const { return mWindows.size(); }
 
     void locateResources() CV_OVERRIDE
@@ -244,10 +268,9 @@ struct Application : public OgreBites::ApplicationContext, public OgreBites::Inp
         ResourceGroupManager& rgm = ResourceGroupManager::getSingleton();
         rgm.createResourceGroup(RESOURCEGROUP_NAME);
 
-        for (size_t i = 0; i < _extraResourceLocations.size(); i++)
+        for (auto loc :_extraResourceLocations)
         {
-            String loc = _extraResourceLocations[i];
-            String type = StringUtil::endsWith(loc, ".zip") ? "Zip" : "FileSystem";
+            const char* type = StringUtil::endsWith(loc, ".zip") ? "Zip" : "FileSystem";
 
             if (!FileSystemLayer::fileExists(loc))
             {
@@ -274,9 +297,10 @@ class WindowSceneImpl : public WindowScene
     Root* root;
     SceneManager* sceneMgr;
     SceneNode* camNode;
-    RenderWindow* rWin;
+    RenderTarget* rWin;
     Ptr<OgreBites::CameraMan> camman;
     Ptr<Rectangle2D> bgplane;
+    std::unordered_map<AnimationState*, Controller<Real>*> frameCtrlrs;
 
     Ogre::RenderTarget* depthRTT;
     int flags;
@@ -286,10 +310,10 @@ public:
     {
         if (!app->sceneMgr)
         {
-            flags |= SCENE_SEPERATE;
+            flags |= SCENE_SEPARATE;
         }
 
-        if (flags & SCENE_SEPERATE)
+        if (flags & SCENE_SEPARATE)
         {
             sceneMgr = root->createSceneManager("DefaultSceneManager", title);
             RTShader::ShaderGenerator& shadergen = RTShader::ShaderGenerator::getSingleton();
@@ -319,20 +343,27 @@ public:
         {
             camman.reset(new OgreBites::CameraMan(camNode));
             camman->setStyle(OgreBites::CS_ORBIT);
-#if OGRE_VERSION >= ((1 << 16) | (11 << 8) | 5)
             camman->setFixedYaw(false);
-#else
-            camNode->setFixedYawAxis(true, Vector3::NEGATIVE_UNIT_Y); // OpenCV +Y in Ogre CS
-#endif
         }
 
         if (!app->sceneMgr)
         {
+            CV_Assert((flags & SCENE_OFFSCREEN) == 0 && "off-screen rendering for main window not supported");
+
             app->sceneMgr = sceneMgr;
             rWin = app->getRenderWindow();
             if (camman)
                 app->addInputListener(camman.get());
         }
+        else if (flags & SCENE_OFFSCREEN)
+        {
+            // render into an offscreen texture
+            TexturePtr tex = TextureManager::getSingleton().createManual(
+                title, RESOURCEGROUP_NAME, TEX_TYPE_2D, sz.width, sz.height, 0, PF_BYTE_RGB,
+                TU_RENDERTARGET, NULL, false, flags & SCENE_AA ? 4 : 0);
+            rWin = tex->getBuffer()->getRenderTarget();
+            rWin->setAutoUpdated(false); // only update when requested
+        }
         else
         {
             OgreBites::NativeWindowPair nwin = app->createWindow(title, sz.width, sz.height);
@@ -346,7 +377,7 @@ public:
 
     ~WindowSceneImpl()
     {
-        if (flags & SCENE_SEPERATE)
+        if (flags & SCENE_SEPARATE)
         {
             TextureManager& texMgr =  TextureManager::getSingleton();
 
@@ -362,13 +393,17 @@ public:
             }
         }
 
-        if(_app->sceneMgr == sceneMgr && (flags & SCENE_SEPERATE))
+        if(_app->sceneMgr == sceneMgr && (flags & SCENE_SEPARATE))
         {
             // this is the root window owning the context
             CV_Assert(_app->numWindows() == 1 && "the first OVIS window must be deleted last");
             _app->closeApp();
             _app.release();
         }
+        else
+        {
+            _app->destroyWindow(title);
+        }
     }
 
     void setBackground(InputArray image) CV_OVERRIDE
@@ -444,6 +479,9 @@ public:
         case PF_BYTE_RGBA:
             dst_type = CV_8UC4;
             break;
+#if OGRE_VERSION >= ((1 << 16) | (12 << 8) | 3)
+        case PF_DEPTH32F:
+#endif
         case PF_FLOAT32_R:
             dst_type = CV_32F;
             break;
@@ -478,10 +516,7 @@ public:
     {
         // hide background plane
         bgplane->setVisible(false);
-
-        // BGRA as uchar
-        ColourValue _color = ColourValue(color[2], color[1], color[0], color[3]) / 255;
-        rWin->getViewport(0)->setBackgroundColour(_color);
+        rWin->getViewport(0)->setBackgroundColour(convertColor(color));
     }
 
     void createEntity(const String& name, const String& meshname, InputArray tvec, InputArray rot) CV_OVERRIDE
@@ -495,7 +530,8 @@ public:
         node->attachObject(ent);
     }
 
-    void removeEntity(const String& name) CV_OVERRIDE {
+    void removeEntity(const String& name) CV_OVERRIDE
+    {
         SceneNode& node = _getSceneNode(sceneMgr, name);
         node.getAttachedObject(name)->detachFromParent();
 
@@ -544,7 +580,6 @@ public:
                            const Scalar& specularColour) CV_OVERRIDE
     {
         Light* light = sceneMgr->createLight(name);
-        light->setDirection(Vector3::NEGATIVE_UNIT_Z);
         // convert to BGR
         light->setDiffuseColour(ColourValue(diffuseColour[2], diffuseColour[1], diffuseColour[0]));
         light->setSpecularColour(ColourValue(specularColour[2], specularColour[1], specularColour[0]));
@@ -576,6 +611,82 @@ public:
         node.setPosition(t);
     }
 
+	void getEntityPose(const String& name ,OutputArray R, OutputArray tvec, bool invert) CV_OVERRIDE
+    {
+		SceneNode* node = sceneMgr->getEntity(name)->getParentSceneNode();
+        Matrix3 _R;
+        // toOGRE.Inverse() == toOGRE
+        (node->getOrientation()*toOGRE).ToRotationMatrix(_R);
+
+        if (invert)
+        {
+            _R = _R.Transpose();
+        }
+
+        if (tvec.needed())
+        {
+            Vector3 _tvec = node->getPosition();
+
+            if (invert)
+            {
+                _tvec = _R * -_tvec;
+            }
+
+            Mat_<Real>(3, 1, _tvec.ptr()).copyTo(tvec);
+        }
+
+        if (R.needed())
+        {
+            Mat_<Real>(3, 3, _R[0]).copyTo(R);
+        }
+    }
+
+    void getEntityAnimations(const String& name, std::vector<String>& out) CV_OVERRIDE
+    {
+        SceneNode& node = _getSceneNode(sceneMgr, name);
+        Entity* ent = dynamic_cast<Entity*>(node.getAttachedObject(name));
+        CV_Assert(ent && "invalid entity");
+        for (auto const& anim : ent->getAllAnimationStates()->getAnimationStates())
+        {
+            out.push_back(anim.first);
+        }
+    }
+
+    void playEntityAnimation(const String& name, const String& animname, bool loop = true) CV_OVERRIDE
+    {
+        SceneNode& node = _getSceneNode(sceneMgr, name);
+        Entity* ent = dynamic_cast<Entity*>(node.getAttachedObject(name));
+        CV_Assert(ent && "invalid entity");
+        AnimationState* animstate = ent->getAnimationState(animname);
+
+        animstate->setTimePosition(0);
+        animstate->setEnabled(true);
+        animstate->setLoop(loop);
+
+        if (frameCtrlrs.find(animstate) != frameCtrlrs.end()) return;
+        frameCtrlrs.insert({
+            animstate,
+            Ogre::ControllerManager::getSingleton().createFrameTimePassthroughController(
+                Ogre::AnimationStateControllerValue::create(animstate, true)
+            )
+        });
+    }
+
+    void stopEntityAnimation(const String& name, const String& animname) CV_OVERRIDE
+    {
+        SceneNode& node = _getSceneNode(sceneMgr, name);
+        Entity* ent = dynamic_cast<Entity*>(node.getAttachedObject(name));
+        CV_Assert(ent && "invalid entity");
+        AnimationState* animstate = ent->getAnimationState(animname);
+
+        if (!animstate->getEnabled()) return;
+
+        animstate->setEnabled(false);
+        animstate->setTimePosition(0);
+        Ogre::ControllerManager::getSingleton().destroyController(frameCtrlrs[animstate]);
+        frameCtrlrs.erase(animstate);
+    }
+
     void setEntityProperty(const String& name, int prop, const String& value) CV_OVERRIDE
     {
         CV_Assert(prop == ENTITY_MATERIAL);
@@ -598,12 +709,28 @@ public:
 
     void setEntityProperty(const String& name, int prop, const Scalar& value) CV_OVERRIDE
     {
-        CV_Assert(prop == ENTITY_SCALE);
         SceneNode& node = _getSceneNode(sceneMgr, name);
-        node.setScale(value[0], value[1], value[2]);
+        switch(prop)
+        {
+        case ENTITY_SCALE:
+        {
+            node.setScale(value[0], value[1], value[2]);
+            break;
+        }
+        case ENTITY_ANIMBLEND_MODE:
+        {
+            Entity* ent = dynamic_cast<Entity*>(node.getAttachedObject(name));
+            CV_Assert(ent && "invalid entity");
+
+            ent->getSkeleton()->setBlendMode(static_cast<Ogre::SkeletonAnimationBlendMode>(int(value[0])));
+            break;
+        }
+        default:
+            CV_Error(Error::StsBadArg, "unsupported property");
+        }
     }
 
-    void getEntityProperty(const String& name, int prop, OutputArray value)
+    void getEntityProperty(const String& name, int prop, OutputArray value) CV_OVERRIDE
     {
         SceneNode& node = _getSceneNode(sceneMgr, name);
         switch(prop)
@@ -704,11 +831,14 @@ public:
         ndc = (2 * f * n) / ndc;
     }
 
+    void update()
+    {
+        rWin->update(false);
+    }
+
     void fixCameraYawAxis(bool useFixed, InputArray _up) CV_OVERRIDE
     {
-#if OGRE_VERSION >= ((1 << 16) | (11 << 8) | 5)
         if(camman) camman->setFixedYaw(useFixed);
-#endif
 
         Vector3 up = Vector3::NEGATIVE_UNIT_Y;
         if (!_up.empty())
@@ -783,9 +913,33 @@ public:
 
         camNode->lookAt(tgt->_getDerivedPosition() + _offset, Ogre::Node::TS_WORLD);
     }
+
+	void setEntityLookAt(const String& origin, const String& target, InputArray offset) CV_OVERRIDE
+    {
+		SceneNode* orig = sceneMgr->getEntity(origin)->getParentSceneNode();
+
+		Vector3 _offset = Vector3::ZERO;
+
+        if (!offset.empty())
+        {
+            offset.copyTo(Mat_<Real>(3, 1, _offset.ptr()));
+        }
+
+		if(target.compare("") != 0){
+        SceneNode* tgt = sceneMgr->getEntity(target)->getParentSceneNode();
+		orig->lookAt(tgt->_getDerivedPosition() + _offset, Ogre::Node::TS_WORLD, Ogre::Vector3::UNIT_Z);
+		}else{
+			orig->lookAt(_offset, Ogre::Node::TS_WORLD, Ogre::Vector3::UNIT_Z);
+		}
+    }
+
 };
 
-CV_EXPORTS_W void addResourceLocation(const String& path) { _extraResourceLocations.push_back(path); }
+CV_EXPORTS_W void addResourceLocation(const String& path)
+{
+    CV_Assert(!_app && "must be called before the first createWindow");
+    _extraResourceLocations.insert(Ogre::StringUtil::normalizeFilePath(path, false));
+}
 
 Ptr<WindowScene> createWindow(const String& title, const Size& size, int flags)
 {
@@ -821,8 +975,8 @@ void setMaterialProperty(const String& name, int prop, const Scalar& val)
     CV_Assert(mat);
 
     Pass* rpass = mat->getTechniques()[0]->getPasses()[0];
-    ColourValue col;
 
+    ColourValue col;
     switch (prop)
     {
     case MATERIAL_POINT_SIZE:
@@ -835,17 +989,16 @@ void setMaterialProperty(const String& name, int prop, const Scalar& val)
         rpass->setSceneBlending(SBT_TRANSPARENT_ALPHA);
         rpass->setDepthWriteEnabled(false);
         break;
+    case MATERIAL_DIFFUSE:
+        col = convertColor(val);
+        col.a = rpass->getDiffuse().a;
+        rpass->setDiffuse(col);
+        break;
     case MATERIAL_EMISSIVE:
-        col = ColourValue(val[2], val[1], val[0]) / 255; // BGR as uchar
-        col.saturate();
-        rpass->setEmissive(col);
+        rpass->setEmissive(convertColor(val));
         break;
     case MATERIAL_LINE_WIDTH:
-#if OGRE_VERSION >= ((1 << 16) | (11 << 8) | 2)
         rpass->setLineWidth(val[0]);
-#else
-        CV_Error(Error::StsError, "needs OGRE 1.11.2+ for this");
-#endif
         break;
     default:
         CV_Error(Error::StsBadArg, "invalid or non Scalar property");
